@@ -471,56 +471,17 @@ else                                     nui['nugget']=nuisance['nugget']
                   }     
   if(sas) { if(bivariate) {}
                         else {
-                          correlation=correlation*(1-nuisance['nugget'] )
-                          corr=correlation
-                          d=as.numeric(nuisance['tail']); 
-                          e=as.numeric(nuisance['skew']); 
-                          sill=as.numeric(nuisance['sill'])
-                  
-    mm=sinh(e/d)*exp(0.25)*(besselK(.25,(d+1)/(2*d))+besselK(.25,(1-d)/(2*d)))/(sqrt(8*pi))
-    vs=cosh(2*e/d)*exp(0.25)*(besselK(.25,(d+2)/(2*d))+besselK(0.25,(2-d)/(2*d)))/(sqrt(32*pi))-0.5-mm^2
-####### starting extra functions
-c1<-function(e,d,n,r){
-   U=c(0.5-0.5/d,-0.5/d);L=c(1-1/d,0.5-0.5/d-n/2+r)
-   res=exp(-e/d)*2^(-0.5+1.5/d+n/2-r)*gamma(0.5+0.5/d+n/2-r)*hypergeo::genhypergeo(U=U, L=L,0.5)
-   return(res)
-  }
-c2<-function(e,d,n,r){
-   U=c(0.5+0.5/d,0.5/d);L=c(1+1/d,0.5+0.5/d-n/2+r)
-   res=exp(-e/d)*2^(-0.5-1.5/d+n/2-r)*gamma(0.5-0.5/d+n/2-r)*hypergeo::genhypergeo(U=U, L=L,0.5)
-   return(res)
-  }
-c3<-function(e,d,n,r){
-   U=c(0.5+n/2-r,1+n/2-r);L=c(1.5-0.5/d+n/2-r,1.5+0.5/d+n/2-r)
-   r1=exp(-e/d)*pi*gamma(1+n-2*r)*hypergeo::genhypergeo(U=U, L=L,0.5)
-   r2=d*gamma(1.5-0.5/d+n/2-r)*gamma(1.5+0.5/d+n/2-r)
-   return(r1/r2)
-   }
-I1<-function(e,d,n,r){
-   a1=cosh(2*e/d)+sinh(2*e/d);  a2=pracma::sec(0.5*pi/d-0.5*n*pi+pi*r)+pracma::sec(0.5*pi/d+0.5*n*pi-pi*r)*a1; a3=pracma::sec(0.5*pi/d+0.5*n*pi-pi*r)+pracma::sec(0.5*pi/d-0.5*n*pi+pi*r)*a1
-   r1=(-1)^(3+n-2*r)*c1(e,d,n,r)-c2(e,d,n,r)+c1(e,d,n,r)*a1; r2=(-1)^(2+n-2*r)*c2(e,d,n,r)*a1+2^(-2-n+2*r)*c3(e,d,n,r)*a2; r3=-(-0.5)^(2+n-2*r)*c3(e,d,n,r)*a3
-  return(r1+r2+r3)
-     }
-SS<-Vectorize(I1, c("r"))
-coef<-function(e,d,N){
-  mat=NULL;n=1
-  while(n<=N){
-   r=as.vector(seq(0,trunc(n/2),1))
-   res=factorial(n)*SS(e,d,n,r)*(-0.5)^r/(2*sqrt(2*pi)*factorial(n-2*r)*factorial(r))
-   mat=c(mat,sum(res));n=n+1}
-return(mat)}
-CC<-Vectorize(coef, c("N"))
-corrsas<-function(e,d,N,vv,rho1){
-  mat=NULL;  j=1
-  while(j<=N){
-     A=CC(e,d,j)^2*rho1^(seq(1:j))/factorial(1:j)
-     mat=sum(A)/vv;j=j+1}
-    return(mat)}
-CorrSAS<-Vectorize(corrsas, c("rho1"))
-##########
-corr=CorrSAS(e,d,4,vs,corr)
+         
+  nugget <- as.numeric(nuisance['nugget'])
+  d <- as.numeric(nuisance['tail'])     # deve essere > 0
+  e <- as.numeric(nuisance['skew'])     # può essere qualsiasi valore
+  sill <- as.numeric(nuisance['sill'])
+  correlation=correlation*(1-as.numeric(nuisance['nugget'] ))
 
-covariance=sill*vs*corr;variogram=sill*vs*(1-corr)  
+mm=sinh(e/d)*exp(0.25)*(besselK(.25,(d+1)/(2*d))+besselK(.25,(1-d)/(2*d)))/(sqrt(8*pi))
+  vv=cosh(2*e/d)*exp(0.25)*(besselK(.25,(d+2)/(2*d))+besselK(0.25,(2-d)/(2*d)))/(sqrt(32*pi))-0.5-mm^2
+  corr=corrsas(correlation, e, d)
+covariance=sill*vv*corr;variogram=sill*vv*(1-corr)  
                         }
           }
 ##########################################
@@ -728,10 +689,6 @@ if(ispatim){
                      xlab="Distance", ylab="Covariance",...)
                 if(show.range) abline(v=Range)}}
          }
-
-
-
-
 OLS=NULL
 ########################### 
 ########################### 
